# -*- coding: utf-8 -*-

from em_tasks import logger
from em_tasks.store.store_util import StoreUtil
from em_tasks.spree.product_util import ProductUtil
from em_tasks.utils.product_converters import AmzProductConverter
from em_tasks.utils.blacklist_filter import BlacklistFilter
from em_tasks.utils.pipelines.amz_products_filter import AmzProductsFilterPipeline
from em_tasks.store.models import *


class StoreAmzProductsFilter():
  def __init__(self, store_code, marketplace, product_service, filter_conds = None):
    self.store = StoreUtil.get_store_by_code(store_code)
    if not self.store:
      return

    credential = json.loads(self.store.api_credential)
    self.product_util = ProductUtil(credential['endpoint'], credential['api_key'], credential.get('api_version', 'v1'))

    self.blacklist_filter = BlacklistFilter()

    self.marketplace = marketplace.lower()
    self.pc = AmzProductConverter(self.marketplace)
    self.product_service = product_service
    self.indice_name = 'amz_products_api_{}'.format(self.marketplace)

    dft_filter_conds = {
      'sales_rank_min': 5000000,
      'weight_max': 2.5,
      'dimension_max': 20,
      'export': False
    }

    self.filter_conds = dft_filter_conds
    if filter_conds:
      self.filter_conds.update(filter_conds)

    if self.filter_conds['export']:
      self.filter_conds['sales_rank_min'] = None

    self.products_filter_pipeline = AmzProductsFilterPipeline(
      self.marketplace, self.filter_conds['weight_max'],
      self.filter_conds['dimension_max'], self.filter_conds['sales_rank_min'],
      self.filter_conds['export'])

  def run(self):
    prods_buf = []
    batch_size = 500
    for prod in self.product_util.get_amz_products(self.marketplace):
      source_product_id = prod.get('source_product_id', prod.get('asin', None))
      if not source_product_id:
        continue

      product_id = prod.get('product_id', None)
      if not product_id:
        continue

      prods_buf.append({'product_id': product_id, 'source_product_id': source_product_id})
      if len(prods_buf) < batch_size:
        continue

      result = self.filter_products(prods_buf)
      if result:
        filtered_products = {pid:entry for pid, entry in result.items() if not entry['passed']}
        if filtered_products:
          yield filtered_products

      prods_buf = []

    if prods_buf:
      result = self.filter_products(prods_buf)
      if result:
        filtered_products = {pid:entry for pid, entry in result.items() if not entry['passed']}
        if filtered_products:
          yield filtered_products

  def filter_products(self, products):
    result = dict()

    asins = [product['source_product_id'] for product in products]
    prods = self.product_service.search_products(self.indice_name, asins)
    if not prods:
      logger.error("[ProductServiceError] No products loaded")
      return

    for product in products:
      asin = product['source_product_id']
      product_id = product['product_id']

      prod = prods.get(asin, None)
      if not prod:
        continue

      r = self.products_filter_pipeline.check_product(prod)
      passed = r.get('passed', True)
      reason = r.get('reason', '')
      if not passed:
        logger.debug('[ProductFiltered] ASIN: %s, Reason: Unuploadable - %s', asin, reason)
        result[product_id] = {'asin': asin, 'passed': False, 'reason': '[Unuploadable] {}'.format(reason)}
        continue

      r = self.check_blacklist(prod)
      passed = r.get('passed', True)
      reason = r.get('reason', '')
      if not passed:
        logger.debug('[ProductFiltered] ASIN: %s, Reason: Blacklisted - %s', asin, reason)
        result[product_id] = {'asin': asin, 'passed': False, 'reason': '[Blacklisted] {}'.format(reason)}
        continue

      result[product_id] = {'asin': asin, 'passed': True, 'reason': ''}

    return result

  def check_blacklist(self, product):
    blacklisted = {'passed': True, 'reason': ''}
    s = '{}-{}'.format(product.get('title', '').lower(), self.pc.get_brand(product).lower())
    result = self.blacklist_filter.is_blacklisted(s)
    if result:
      blacklisted = {'passed': False, 'reason': '[BlacklistKeyword] {}'.format(result)}

    return blacklisted
