# -*- coding: utf-8 -*-

from em_tasks import logger
from em_tasks.utils.product_service import ProductService
from em_tasks.store.store_util import StoreUtil
from em_tasks.spree.product_util import ProductUtil
from em_tasks.utils.product_converters import AmzProductConverter
from em_tasks.utils.pipelines.amz_products_filter import AmzProductsFilterPipeline
from em_tasks.utils.blacklist_filter import BlacklistFilter
from em_tasks.store.models import *


class AmzProductsAnalyzer():
  def __init__(self, marketplace, product_service, offer_service, asins_path, output_path, condition='new'):
    self.product_service = product_service
    self.offer_service = offer_service
    self.marketplace = marketplace.lower()
    self.condition = condition
    self.asins_path = asins_path
    self.pc = AmzProductConverter(self.marketplace)
    self.output_path = os.path.abspath(os.path.expanduser(output_path))
    output_dir = os.path.dirname(self.output_path)
    if not os.path.isdir(output_dir):
      os.makedirs(output_dir)

    self.stats = {
      'products_cnt': 0,
      'uploadable_cnt': 0,
      'has_info_cnt': 0,
      'no_info_cnt': 0,
      'info_filtered_cnt': 0,
      'has_offers_cnt': 0,
      'no_offers_cnt': 0,
      'has_offer_cnt': 0,
      'offer_filtered_cnt': 0
    }

    self.sales_rank_min = 1500000
    if self.marketplace == 'us':
      self.sales_rank_min = 500000

    self.weight_max = 2.5
    self.dimension_max = 20
    self.blacklist_filter = BlacklistFilter()

    self.products_filter_pipeline = AmzProductsFilterPipeline(self.marketplace, self.weight_max, self.dimension_max, self.sales_rank_min)
    self.product_indice_name = 'amz_products_api_{}'.format(marketplace.lower())
    self.headers = ['asin', 'uploadable', 'has_product_info', 'filtered', 'filtered_reason', 'has_offers', 'has_offer', 'offer']

  def run(self):
    batch_size = 500
    asin_dict = dict()
    try:
      self.output_fh = open(self.output_path, 'w+', encoding='utf-8', errors='ignore')
      self.output_fh.write("\t".join(self.headers))
      self.output_fh.write("\n")
      with open(self.asins_path, "r") as file:
        for asin in file:
          self.stats['products_cnt'] += 1

          asin_dict[asin.strip()] = None
          if len(asin_dict) < batch_size:
            continue
          self.process_products(list(asin_dict.keys()))
          asin_dict = {}

        if asin_dict:
          self.process_products(list(asin_dict.keys()))
          asin_dict = {}
    finally:
      self.output_fh.close()

  def get_stats(self):
    return self.stats

  def check_blacklist(self, product):
    blacklisted = {'passed': True, 'reason': ''}

    s = '{}-{}'.format(product.get('title', '').lower(), self.pc.get_brand(product).lower())
    result = self.blacklist_filter.is_blacklisted(s)
    if result:
      blacklisted = {'passed': False, 'reason': '[BlacklistKeyword] {}'.format(result)}

    return blacklisted

  def process_products(self, asins):
    records = []
    result = {asin:{} for asin in asins}

    products_info = self.product_service.search_products(self.product_indice_name, asins)
    for asin in asins:
      product_info = products_info.get(asin, None) if products_info else None
      if product_info:
        result[asin]['has_product_info'] = True
        res = self.products_filter_pipeline.check_product(product_info)
        passed = res.get('passed', True)
        reason = res.get('reason', '')
        if passed:
          result[asin]['filtered'] = False
          result[asin]['filtered_reason'] = None
        else:
          result[asin]['filtered'] = True
          result[asin]['filtered_reason'] = reason

      else:
        result[asin]['has_product_info'] = False

    offers = dict()
    offers_result = self.offer_service.analyze_offers(self.marketplace, asins, self.condition)
    for asin, offer_result in offers_result.items():
      result[asin].update(offer_result)

    for asin, res in result.items():
      item = {
        'asin': asin,
        'has_product_info': res.get('has_product_info', False),
        'filtered': res.get('filtered', False),
        'filtered_reason': res.get('filtered_reason', None),
        'has_offers': res.get('has_offers', False),
        'has_offer': res.get('has_offer', False),
        'offer': res.get('offer', None)
      }

      if item['has_product_info']:
        self.stats['has_product_info'] += 1

        if item['filtered']:
          self.stats['info_filtered_cnt'] += 1
      else:
        self.stats['no_info_cnt'] += 1

      if item['has_offers']:
        self.stats['has_offers_cnt'] += 1

        if item['has_offer']:
          self.stats['has_offer_cnt'] += 1
        else:
          self.stats['offer_filtered_cnt'] += 1
      else:
        self.stats['no_offers_cnt'] += 1

      item['uploadable'] = item['has_product_info'] and not item['filtered'] and item['has_offer']
      records.append([str(item[header]) for header in self.headers])

      if item['uploadable']:
        self.stats['uploadable_cnt'] += 1

    self.output_fh.write("\n".join(["\t".join(record) for record in records]))
    self.output_fh.write("\n")
