import json
import datetime
import pdb

import sentry_sdk
from dropshipping.utils.offer_filters import LowestOfferListingOfferFilter

from em_tasks import logger
from em_tasks.spree.product_util import ProductUtil
from em_tasks.store.store_util import StoreUtil
from em_tasks.tasks.spapi_update_item_offers_task import SpapiUpdateItemOffersTask
from em_tasks.spapi import exceptions_to_retry, exceptions_not_retry
from em_tasks.store.models import *


class SpapiRepricer():
  def __init__(self, store_code, spapi, src_marketplace, filter_cond, offer_service, price_caculator):
    self.condition = 'new'

    self.store = StoreUtil.get_store_by_code(store_code)
    if not self.store:
      return

    self.spapi = spapi
    self.offer_service = offer_service
    self.filter_cond = filter_cond
    self.offer_filter = LowestOfferListingOfferFilter(self.filter_cond)

    credential = json.loads(self.store.api_credential)
    self.product_util = ProductUtil(credential['endpoint'], credential['api_key'], credential.get('api_version', 'v1'))

    self.src_marketplace = src_marketplace.lower()
    self.source = 'AMZ_{}'.format(src_marketplace.upper())
    self.price_calc = price_caculator
    self.stats = {'products_cnt': 0, 'expired_cnt': 0, 'failed_cnt': 0, 'in_stock': 0, 'out_of_stock': 0}

  def run(self):
    orders = []
    prods = {}

    earliest_purchase_date = datetime.datetime.utcnow() - datetime.timedelta(days=3)
    batch_size = 500
    page = 1
    query = OrderItem.select().where((OrderItem.source == self.source) & (OrderItem.s_purchase_date > earliest_purchase_date))
    total_cnt = query.count()

    sql, param = query.sql()
    logger.info("[SQL] {}".format(sql.replace("?","{}").format(*param)))

    logger.info("[OrdersCnt] {}".format(total_cnt))

    pages = int((total_cnt + batch_size) / batch_size)
    while page <= pages:
      for oi in query.paginate(page, batch_size):
        if (not oi.approved_at) and (oi.risky or oi.risky_action != 'accept'):
          continue

        if oi.s_product_id in prods:
          continue

        prods[oi.s_product_id] = {
          'product_id': oi.s_product_id,
          'source': self.source,
          'source_product_id': oi.source_id,
          'handle': oi.s_url.split('/').pop(),
          'variants': [{'variant_id': oi.s_variant_id}],
        }

      logger.info("[OrdersLoaded] Page: {}".format(page))

      page+= 1

    buf = {}
    buf_size = 20
    for prod_id, prod in prods.items():
      buf[prod_id] = prod
      if len(buf) < buf_size:
        continue

      self.process_products(buf)
      buf.clear()

    if buf:
      self.process_products(buf)

  def process_products(self, prods):
    offers = dict()

    asins = [prod['source_product_id'] for prod in prods.values()]
    condition = 'new'

    task = SpapiUpdateItemOffersTask(self.spapi, self.offer_service, self.src_marketplace, asins, condition)
    spapi_offers = task.run()

    src_offers = dict()
    for asin, converted_offer_list in spapi_offers.items():
      if not converted_offer_list or not converted_offer_list.get('offers', None):
        src_offers[asin] = None
      else:
        src_offers[asin] = self.offer_filter.filter(converted_offer_list.get('offers'))

    logger.info("[SpapiOffers] {}".format(src_offers))

    for asin, src_offer in src_offers.items():
      offers[asin] = {'asin': asin, 'src_offer': src_offer}
      offer = self.price_calc.calc_offer(src_offer)
      if offer:
        offers[asin].update(offer)
      else:
        offers[asin] = offer

    for _, prod in prods.items():
      asin = prod['source_product_id']
      if asin in offers:
        offer = offers[asin]
        prod['offer'] = offer

    try:
      store_offers = self.product_util.set_products_offer(prods, None)
    except Exception as e:
      try:
        sentry_sdk.capture_exception(e)
      except:
        pass
