import datetime
import pdb

import dateutil
import dateparser
from tracking_tracker.ups_api import UPSApi

from em_tasks import logger


class TrackingParser():
  def parse(self, tracking_number, tracking):
    raise NotImplementedError()

  def parse_timelines(self, tracking):
    raise NotImplementedError()


class FedexTrackingParser(TrackingParser):
  def parse(self, tracking_number, tracking):
    if not tracking or 'output' not in tracking or 'completeTrackResults' not in tracking['output'] or not tracking['output']['completeTrackResults']:
      error ='[InvalidResponse] {}'.format(tracking)
      return {'error': error}

    track_results = None
    for complete_track_result in tracking['output']['completeTrackResults']:
      tn =  complete_track_result.get('trackingNumber', None)
      if tn == tracking_number:
        track_results = complete_track_result.get('trackResults', None)
        break

    if not track_results:
      error = '[NoTrackResults] {}'.format(tracking_number)
      return {'error': error}

    valid_track_result = None
    for track_result in track_results:
      if 'error' in track_result and track_result['error'].get('code', None) == 'TRACKING.TRACKINGNUMBER.NOTFOUND':
        return {'error': track_result['error'].get('code', None)}

      valid_track_result = track_result
      break

    if not valid_track_result:
      error = '[NoValidTrackResult] {}'.format(tracking)
      return {'error': error}

    timelines = self.parse_timelines(valid_track_result)
    delivered = any([timeline['status'].lower().find('delivered') != -1 for timeline in timelines if timeline['status']])
    status = 'IN_TRANSIT'
    if delivered:
      status = 'DELIVERED'

    last_update_date = None
    last_update_status = None
    if timelines:
      for last_update in timelines:
        date_s = last_update['date']
        parsed_date = dateparser.parse(date_s)
        if parsed_date:
          last_update_date = parsed_date.strftime('%Y-%m-%d')
        else:
          last_update_date = date_s

        last_update_status = last_update.get('status_detail', None) or last_update.get('status', None)

        if last_update_status or last_update_date:
          break

    # if not timelines or not last_update_date:
    #   pdb.set_trace()

    return {
      'tracking_number': tracking_number,
      'delivered': delivered,
      'last_update_date': last_update_date,
      'last_update_status': last_update_status,
    }

  def parse_timelines(self, tracking):
    if not tracking or 'scanEvents' not in tracking or not tracking['scanEvents']:
      return []

    timelines = []
    d = dict()
    shipment = tracking['scanEvents']
    for event in tracking['scanEvents']:
      date_s = event['date']
      parsed_date = None
      try:
        parsed_date = dateutil.parser.parse(date_s)
      except:
        try:
          parsed_date = dateparser.parse(date_s)
        except Exception as e:
          # logger.exception(e)
          parsed_date = None
      if parsed_date:
        date_s = parsed_date.strftime('%m/%d/%Y')

      scan_location = event.get('scanLocation', d)
      location = '{}, {} {}'.format(scan_location.get('city', ''), scan_location.get('stateOrProvinceCode', ''), scan_location.get('postalCode', ''))

      status = event.get('derivedStatus', event.get('derivedStatus', ''))
      description = event.get('eventDescription', '')

      timelines.append({
        'status': status,
        'status_detail': description,
        'location': location,
        'date': date_s
      })

    return timelines


class UPSTrackingParser(TrackingParser):
  def parse(self, tracking_number, tracking):
    if not tracking or 'trackResponse' not in tracking:
      error = '[InvalidResponse] {}'.format(tracking)
      return {'error', error}

    timelines = self.parse_timelines(tracking['trackResponse'])
    delivered = any([timeline['status'].lower().find('delivered') != -1 for timeline in timelines if timeline['status']])
    status = 'IN_TRANSIT'
    if delivered:
      status = 'DELIVERED'

    last_update_date = None
    last_update_status = None
    if timelines:
      for last_update in timelines:
        date_s = last_update['date']
        parsed_date = dateparser.parse(date_s)
        if parsed_date:
          last_update_date = parsed_date.strftime('%Y-%m-%d')
        else:
          last_update_date = date_s
        last_update_status = last_update.get('status_detail', None) or last_update.get('status', None)

        if last_update_status or last_update_date:
          break

    # if not timelines or not last_update_status:
    #   pdb.set_trace()

    return {
      'tracking_number': tracking_number,
      'delivered': delivered,
      'last_update_date': last_update_date,
      'last_update_status': last_update_status,
    }

  def parse_timelines(self, tracking):
    if not tracking or 'shipment' not in tracking or not tracking['shipment']:
      return []

    timelines = []
    d = dict()
    shipment = tracking['shipment'][0]
    if 'package' not in shipment or not shipment['package']:
      return []
    activities = shipment['package'][0]['activity']
    for activity in activities:
      date_s = '{}T{}'.format(activity['date'], activity['time'])
      try:
        parsed_date = dateutil.parser.parse(date_s)
      except Exception as e:
        try:
          parsed_date = dateparser.parse(date_s)
        except Exception as e:
          # logger.exception(e)
          parsed_date = None
      if parsed_date:
        date_s = parsed_date.strftime('%m/%d/%Y')
      else:
        date_s = activity['date']

      address = activity['location'].get('address', d)
      if address:
        city = address.get('city', '')
        state = address.get('stateProvince', '')
        if state:
          location = '{}, {} {}'.format(
            city, state, address.get('postalCode', '')).strip()
        else:
          location = city.strip()
      else:
        location = ''

      state = activity['status']
      status = UPSApi.get_status_by_processing_code(state.get('statusCode', ''))
      if not status:
        if state['description'].lower().find('delivered') != -1:
          status = 'Delivered'
      timelines.append({
        'status': status or '',
        'status_detail': state['description'],
        'location': location,
        'date': date_s
      })

    return timelines


class USPSTrackingParser(TrackingParser):
  def parse(self, tracking_number, tracking):
    if not tracking or not tracking.result:
      error = '[InvalidResponse] {}'.format(tracking)
      return {'error': error}

    d = dict()
    track_error = tracking.result.get('TrackResponse', d).get('TrackInfo', d).get('Error', None)
    if track_error:
      error = '[NoValidTrackResult] {}'.format(tracking)
      return {'error': error}

    timelines = self.parse_timelines(tracking)
    delivered = any([timeline['status'].lower().find('delivered') != -1 for timeline in timelines if timeline['status']])
    status = 'IN_TRANSIT'
    if delivered:
      status = 'DELIVERED'

    last_update_date = None
    last_update_status = None
    if timelines:
      for last_update in timelines:
        date_s = last_update['date']
        parsed_date = dateparser.parse(date_s)
        if parsed_date:
          last_update_date = parsed_date.strftime('%Y-%m-%d')
        else:
          last_update_date = date_s

        last_update_status = last_update.get('status_detail', None) or last_update.get('status', None)

        if last_update_status or last_update_date:
          break

    return {
      'tracking_number': tracking_number,
      'delivered': delivered,
      'last_update_date': last_update_date,
      'last_update_status': last_update_status
    }

  def parse_timelines(self, tracking):
    d = dict()
    events = tracking.result.get('TrackResponse', d).get('TrackInfo', d).get('TrackDetail', [])
    summary = tracking.result.get('TrackResponse', d).get('TrackInfo', d).get('TrackSummary', d)
    if not events:
      if not summary:
        return []

      events = [summary]
    else:
      if isinstance(events, dict):
        events = [events]

      last_event = events[0]
      last_event_desc = last_event.get('Event', '')

      summary_desc = summary.get('Event', '')
      delivered = summary_desc.lower().find('delivered') != -1
      returned = summary_desc.lower().find('return') != -1
      if delivered:
        if last_event_desc.lower().find('delivered') == -1:
          events.insert(0, summary)

      if returned:
        if last_event_desc.lower().find('return') == -1:
          events.insert(0, summary)

    # if isinstance(events, dict):
    #   events = [events]

    timelines = []
    for event in events:
      try:
        date_s = event['EventDate']
        parsed_date = None
        try:
          parsed_date = dateutil.parser.parse(date_s)
        except:
          try:
            parsed_date = dateparser.parse(date_s)
          except Exception as e:
            # logger.exception(e)
            parsed_date = None
        if parsed_date:
          date_s = parsed_date.strftime('%m/%d/%Y')

        location_parts = [part for part in [event.get('EventCity', None), event.get('EventState', None), event.get('EventZIPCode', None)] if part]
        location = ','.join(location_parts)

        description = event.get('Event', '')
        status = 'IN_TRANSIT'
        if description.lower().find('delivered') != -1:
          status = 'Delivered'

        timelines.append({
          'status': status,
          'status_detail': description,
          'location': location,
          'date': date_s
        })
      except Exception as e:
        logger.exception(e)

    return timelines

